"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_js_1 = __importDefault(require("crypto-js"));
const keySize = 256;
const iterations = 100;
class Vault {
    constructor(store) {
        this._store = store;
    }
    keys() {
        return this._store.keys();
    }
    contains(key) {
        return this.keys().indexOf(key) >= 0;
    }
    set(key, value, password) {
        if (this.contains(key))
            throw new Error(`${key} already present`);
        const ciphertext = this.encrypt(JSON.stringify(value), password);
        this._store.set(key, ciphertext);
    }
    get(key, password) {
        if (!this.contains(key))
            throw new Error(`${key} is not present`);
        const transitMessage = this._store.get(key);
        try {
            const data = this.decrypt(transitMessage, password);
            return JSON.parse(data);
        }
        catch (e) {
            throw new Error('Incorrect password');
        }
    }
    remove(key) {
        if (!this.contains(key))
            throw new Error(`${key} is not present`);
        this._store.set(key, undefined);
    }
    // https://github.com/luniehq/cosmos-keys/blob/7d848f325a2f5c32c720f07722b239c6ebded542/src/cosmos-keystore.ts#L144
    encrypt(message, password) {
        const salt = crypto_js_1.default.lib.WordArray.random(128 / 8);
        const key = crypto_js_1.default.PBKDF2(password, salt, {
            keySize: keySize / 32,
            iterations: iterations
        });
        const iv = crypto_js_1.default.lib.WordArray.random(128 / 8);
        const encrypted = crypto_js_1.default.AES.encrypt(message, key, {
            iv: iv,
            padding: crypto_js_1.default.pad.Pkcs7,
            mode: crypto_js_1.default.mode.CBC
        });
        // salt, iv will be hex 32 in length
        // append them to the ciphertext for use  in decryption
        const transitmessage = salt.toString() + iv.toString() + encrypted.toString();
        return transitmessage;
    }
    // https://github.com/luniehq/cosmos-keys/blob/7d848f325a2f5c32c720f07722b239c6ebded542/src/cosmos-keystore.ts#L166
    decrypt(transitMessage, password) {
        const salt = crypto_js_1.default.enc.Hex.parse(transitMessage.substr(0, 32));
        const iv = crypto_js_1.default.enc.Hex.parse(transitMessage.substr(32, 32));
        const encrypted = transitMessage.substring(64);
        const key = crypto_js_1.default.PBKDF2(password, salt, {
            keySize: keySize / 32,
            iterations: iterations
        });
        const decrypted = crypto_js_1.default.AES.decrypt(encrypted, key, {
            iv: iv,
            padding: crypto_js_1.default.pad.Pkcs7,
            mode: crypto_js_1.default.mode.CBC
        }).toString(crypto_js_1.default.enc.Utf8);
        return decrypted;
    }
}
exports.default = Vault;
